
# AURA Player

**AURA Player is a framework agnostic user interface toolbox** for integrating the AURA Radio Software Suite into your website and Content Management System.

These JavaScript components are written in [Svelte](https://svelte.dev/docs) and are reading data from the [AURA API](https://gitlab.servus.at/aura/meta). As a result all components are provided in a lightweight package. They are pluggable into any front-end framework or website.

Some of the components are provided as true [HTML 5 Web Components](https://en.wikipedia.org/wiki/Web_Components), most others are delivered as [Svelte Components](https://svelte.dev/). The latter provide easier integration and styling options and are therefore preferred for future development.

Component features:

* [Cards Design Pattern](https://www.justinmind.com/blog/cards-ui-design/)
* Based on [Material Design](https://material.io/), more specifically [SMUI](https://sveltematerialui.com/)
* [Responsive Design](https://en.wikipedia.org/wiki/Responsive_web_design), supporting a wide variety of mobile and desktop devices
* Solid foundation for [accessibility](https://material.io/design/usability/accessibility.html#understanding-accessibility) requirments
* Themeable and stylable, including dark themes.
* [TypeScript](https://www.typescriptlang.org/) support
* [SCSS](https://sass-lang.com/) support

Provided components:

* (Auto) Radio Player - A minimalist radio player (Important: Currently only integrated with the o94 legacy player popup)
* Programme Information - A Web Component to render radio programme infos in various forms (Currently only integrated with o94)
* Track Service (*WIP*: not up-to-date with latest `engine-api`)
* Service Worker - A HTML 5 `service-worker.js` including a Web Component to load the Service Worker plus sample `pre-cache.json` and `manifest.json` files
* Online Status - A *Toast Notification* component for online/offline status changes
* Host List Component
* Host Detail Component
* Show List Component
* Show Detail Component

> **Disclaimer:** The components of this repository are *Work in Progress* and hence not fully featured nor documented in all aspects. Currently some parts are customized for an Radio Orange 94.0 integration. This component will be generalized in the future upon request from third-parties. In addition Radio ORANGE 94.0 developed a Drupal 9 integration based on the AURA Player components. If you are interested using AURA in conjunction with Drupal, get in touch. Also, these components are using a simulated AURA API for now. This API is hosted at https://prog-info.o94.at/

<!-- TOC -->

1. [AURA Player](#aura-player)
   1. [Website Integration](#website-integration)
   2. [Components](#components)
      1. [Radio Player](#radio-player)
      2. [Programme Information](#programme-information)
      3. [Track Service](#track-service)
      4. [Online Status (Toast Notification)](#online-status-toast-notification)
      5. [Host](#host)
         1. [Host List Component](#host-list-component)
         2. [Host Detail Component](#host-detail-component)
      6. [Show](#show)
         1. [Show List Component](#show-list-component)
         2. [Show Detail Component](#show-detail-component)
      7. [Episode](#episode)
         1. [Episode Detail Component](#episode-detail-component)
   3. [Progressive Web App](#progressive-web-app)
      1. [Service Worker](#service-worker)
      2. [Manifest](#manifest)
      3. [Offline mode](#offline-mode)
         1. [Offline mode with cached pages](#offline-mode-with-cached-pages)
         2. [Offline mode with `offline.html`](#offline-mode-with-offlinehtml)
   4. [Development](#development)
      1. [Requirements](#requirements)
      2. [Install](#install)
      3. [Run](#run)
   5. [Production](#production)
      1. [Link the bundle](#link-the-bundle)
2. [About](#about)

<!-- /TOC -->

## Website Integration

The player components can be integrated to your website with a few simple steps.

## Components

### Radio Player

Renders the Radio Player.

> Note: This component is currently only replicated the Radio ORANGE 94.0 legacy player popup.

```html
<div id="aura-player"></div>
<script>
    /** Inject component into HTML **/
    document.addEventListener("DOMContentLoaded", function (event) {
        new AuraPlayer({
            target: document.getElementById("aura-player"),
            props: {
                renderhtml: true,
                liveurl: "https://o94.at/de/player/live"
            }
        });
    });
</script>
```

### Programme Information

Renders the daily programme info by listing all shows of the selected day.

```html
<div id="aura-programme"></div>
<script>
    /** Inject component into HTML **/
    document.addEventListener("DOMContentLoaded", function (event) {
        new AuraProgramme({
            target: document.getElementById("aura-programme"),
            props: {
                date: "today",
                limitCount: 0,
                urlEpisodeDetail: '/episode-detail.html?id=',
                renderhtml: true
            }
        });
    });
</script>
```

### Track Service

Track Service is a Web Component which displays the track currently played by the radio station. Additionally there's a history of past and future tracks.

Render the component in the desired place of your website like this:

```html
<div id="aura-trackservice"></div>
<script>
    /** Inject component into HTML **/
    document.addEventListener("DOMContentLoaded", function (event) {
        new AuraTrackService({
            target: document.getElementById("aura-trackservice"),
            props: {
                labelTitle: "Which track is playing right now?"
            }
        });
    });
</script>
```

### Online Status (Toast Notification)

Displays a toast notification whether the app moved to an *online* or *offline* state.

```html
<div id="online-status-container"></div>
<script>
    /** Inject component into HTML **/
    document.addEventListener("DOMContentLoaded", function (event) {
        new OnlineStatus({
            target: document.getElementById("online-status-container"),
            props: {
                onlinetext: "You are back online.",
                offlinetext: "You have no connection."
            }
        });
    });
</script>
```

### Host

#### Host List Component

Displays a list of *Host Cards*, including pagination.

```html
<div id="host-list"></div>
<script>
    /** Inject component into HTML **/
    document.addEventListener("DOMContentLoaded", function (event) {
        new AuraHostList({
            target: document.getElementById("host-list"),
            props: {
                renderhtml: true,
                urlHostDetail: "/host-detail.html?id=",
                activeonly: true,
                page: 0,
                limit: 600
            }
        });
    });
</script>
```

#### Host Detail Component

Displays a *Host Detail Card*.

```html
<div id="host-detail"></div>
<script>
    /** Inject component into HTML **/
    document.addEventListener("DOMContentLoaded", function (event) {
        new AuraHostDetail({
            target: document.getElementById("host-detail"),
            props: {
                renderhtml: true,
                hostid: 707050,
                urlShowDetail: "/show-detail?slug="
            }
        });
    });
</script>
```

### Show

#### Show List Component

Displays a list of *Show Card*.

```html
<div id="show-list"></div>
<script>
    /** Inject component into HTML **/
    document.addEventListener("DOMContentLoaded", function (event) {
        new AuraShowList({
            target: document.getElementById("show-list"),
            props: {
                renderhtml: true,
                urlShowDetail: "/show-detail.html?slug=",
                activeonly: true,
                limit: 600
            }
        });
    });
</script>
```

#### Show Detail Component

Displays a *Show Detail Card*.

```html
<div id="show-detail"></div>
<script>
    /** Inject component into HTML **/
    document.addEventListener("DOMContentLoaded", function (event) {
        new AuraShowDetail({
            target: document.getElementById("show-detail"),
            props: {
                renderhtml: true,
                showSlug: "female-pressure",
                labelHosts: "Hosts",
                labelEpisodes: "Episodes",
                urlEpisodeDetail: "/episode-detail.html?id=",
                urlHostDetail: "/host-detail.html?id="
            }
        });
    });
</script>
```

### Episode

#### Episode Detail Component

Displays an *Episode Detail Card*.

```html
<div id="episode-detail"></div>
<script>
		/** Inject component into HTML **/
		document.addEventListener("DOMContentLoaded", function (event) {
			new AuraEpisodeDetail({
				target: document.getElementById("episode-detail"),
				props: {
					renderhtml: true,
					episodeId: randomId,
					labelDisplayShow: "Visit",
					labelPlay: "▶ Play",
					labelLinkArchive: "Open in archive",
					urlShowDetail: "/show-detail.html?slug=",
					playerDomId: "aura-player"
				}
			});
		});
</script>
```

## Progressive Web App

The Player bundle also provides some building blocks to turn your website into a Progressive Web App. Keep in mind, that those are only serving as an entry point. For a fully-fledged PWA you'll need to add your own code and thoughts.

This repository also provides a sample app, allowing you to test the PWA beforehand. To start the app using the development server type:

```bash
./run.sh
```

### Service Worker

A production-ready Service Worker with offline functionality can be found under `public/service-worker.js`. Copy the Service worker to your web server and modify at least following variables:

```javascript
let OFFLINE_URL = "/offline.html"; // Match your offline page URL
let PRE_CACHE = "pre-cache-v1"; // Manually update version or generate one server-site
let PRE_CACHE_URL = "/pre-cache.json" // Match your pre-cache URL
```

To load and install the Service Worker place the following custom tag somewhere on your page:

```html
<aura-service-worker></aura-service-worker>
```

The default component expects the service-service to be served via the URL fragment `/service-worker.js` and a JSON file holding resources to be pre-cached under `/pre-cache.json`. The pre-cached offline page should be served at `/offline.html`.

If you have custom requirements you can overwrite this behaviour by using the following attributes:

```html
<aura-service-worker url="/custom/path/service-worker.js">
</aura-service-worker>
```

The Service Worker is based on a network-first strategy:

1. Incoming requests are first passed to the network
2. If the network is not available it queries the local cache
3. If the resource cannot be found in the cache, the pre-cached `offline.html` is returned

### Manifest

A sample Web Manifest is located in `public/manifest.json`. Copy this file to your web server and modify it according to your needs.

### Offline mode

The app provides multiple fallback stages when offline.

#### Offline mode with cached pages

Pages defined in `pre-cache.json` will be pre-cached upon Service Worker installation and hence are available offline. Copy this file to your web server and modify it according to your needs.

Perform following steps to verify the pre-cache to be working:

1. Start the development server and visit the start page. Do not visit the online page yet.
2. Stop the development server, but do not close the start page.
3. Refresh the start page. It will load as if a connection is available.

#### Offline mode with `offline.html`

To test the pages where the offline page is served as a fallback, do following:

1. Start the development server and visit the start page. Do not visit the `/online.html` page yet, to avoid being cached in the runtime cache.
2. Now the Service Worker is installed, pages in `public/pre-cache.json` plus `public/offline.html` are pre-cached.
3. Stop the development server, but do not close the start page.
4. Visit the URL `/online.html` via the link on the start page. Since this document is not stored in the cache, the default `/offline.html` will be returned.

If these steps do not work as described you'll need to ensure that either your cached is cleared beforehand, or you do increment the cache version in the `pre-cache.json` file.


## Development

### Requirements

* Node (>= 13.0)

### Install

```bash
npm install
```

### Run

To start the development server execute

```bash
./run.sh
```
or

```bash
./run.sh dev
```

## Production

To build the components for production run

```bash
./run.sh build
```

Use the generated components in your own website.

### Link the bundle

Place the JS and CSS files in `trackservice/src/public/build` to the relevant directory of the web-server.

Then include 'em in the template or some specific page of your Radio Stations's website:

```html
<head>
    ...
    <script defer src='/build/aura-player-bundle.js'></script>
    ...
</head>
```

Now you can instantiate any component as outlined above in the [Components](#Components) section.

# About

[<img src="https://gitlab.servus.at/autoradio/meta/-/raw/master/assets/images/aura-logo.png" width="150" />](https://gitlab.servus.at/aura/meta)

AURA stands for Automated Radio and is a swiss army knife for community radio stations. Beside the Engine it provides Steering (Admin Interface for the radio station), Dashboard (Collaborative scheduling and programme coordination), Tank (Audio uploading, pre-processing and delivery). Read more in the [Aura Meta](https://gitlab.servus.at/aura/meta) repository or on the specific project pages.

| [<img src="https://gitlab.servus.at/aura/meta/-/raw/master/assets/images/aura-steering.png" width="150" align="left" />](https://gitlab.servus.at/aura/steering)  |  [<img src="https://gitlab.servus.at/aura/meta/-/raw/master/assets/images/aura-dashboard.png" width="150" align="left" />](https://gitlab.servus.at/aura/dashboard)  |  [<img src="https://gitlab.servus.at/aura/meta/-/raw/master/assets/images/aura-tank.png" width="150" align="left" />](https://gitlab.servus.at/aura/tank) | [<img src="https://gitlab.servus.at/aura/meta/-/raw/master/assets/images/aura-engine.png" width="150" align="left" />](https://gitlab.servus.at/aura/engine)  |
|---|---|---|---|
| [Steering](https://gitlab.servus.at/aura/steering)  | [Dashboard](https://gitlab.servus.at/aura/dashboard)<br/>[Dashboard Clock](https://gitlab.servus.at/aura/dashboard-clock)  | [Tank](https://gitlab.servus.at/aura/tank)  | [Engine](https://gitlab.servus.at/aura/engine)<br/>[Engine Core](https://gitlab.servus.at/aura/engine-core)<br/>[Engine API](https://gitlab.servus.at/aura/engine-api)  |
