import svelte from 'rollup-plugin-svelte';
import sveltePreprocess from 'svelte-preprocess';
import typescript from "@rollup/plugin-typescript";
import commonjs from '@rollup/plugin-commonjs';
import resolve from '@rollup/plugin-node-resolve';
import livereload from 'rollup-plugin-livereload';
import { terser } from 'rollup-plugin-terser';
import postcss from 'rollup-plugin-postcss';
import replace from '@rollup/plugin-replace';
import pkg from "./package.json";


const production = !process.env.ROLLUP_WATCH;

function serve() {
	let server;

	function toExit() {
		if (server) server.kill(0);
	}

	return {
		writeBundle() {
			if (server) return;
			server = require('child_process').spawn('npm', ['run', 'start', '--', '--dev'], {
				stdio: ['ignore', 'inherit', 'inherit'],
				shell: true
			});

			process.on('SIGTERM', toExit);
			process.on('exit', toExit);
		}
	};
}

export default {
	input: 'src/main.ts',
	output: {
		sourcemap: true,
		format: 'iife',
		name: 'app',
		file: 'public/build/aura-player-bundle.js'
	},
	plugins: [
		/** Inject App Version **/
		replace({
			preventAssignment: true,
			APP_VERSION: pkg.version
		}),

		/** Handle HTML5 Web Components **/
		svelte({
			include: /\.wc\.svelte$/,
			preprocess: sveltePreprocess({
				sourceMap: !production,
			}),
			compilerOptions: {
				// enable run-time checks when not in production
				dev: !production,
				customElement: true,
			},
			// store CSS in JavaScript
			emitCss: false
		}),

		/** Handle Svelte Components **/
		svelte({
			exclude: /\.wc\.svelte$/,
			preprocess: sveltePreprocess({
				sourceMap: !production,
			}),
			compilerOptions: {
				// enable run-time checks when not in production
				dev: !production,
				customElement: false,
			},
			// we'll extract any component CSS out into
			// a separate file - better for performance
			emitCss: true
		}),

		// we'll extract any component CSS out into
		// a separate file - better for performance
		// css({ output: 'aura-player-bundle.css' })

		/** SASS Support */
		postcss({
			extract: true,
			minimize: production,
			use: [
			  [
				'sass',
				{
				  includePaths: ['./src', './src/theme', './node_modules'],
				},
			  ],
			],
		  }),

		// If you have external dependencies installed from
		// npm, you'll most likely need these plugins. In
		// some cases you'll need additional configuration -
		// consult the documentation for details:
		// https://github.com/rollup/plugins/tree/master/packages/commonjs
		resolve({
			browser: true,
			dedupe: ['svelte']
		}),
		commonjs(),

		/** TypeScript Support  */
		typeCheck(),
		typescript({ rootDir: './src', sourceMap: !production }),

		// In dev mode, call `npm run start` once
		// the bundle has been generated
		!production && serve(),

		// Watch the `public` directory and refresh the
		// browser on changes when not in production
		!production && livereload('public'),

		// If we're building for production (npm run build
		// instead of npm run dev), minify
		production && terser()
	],
	watch: {
		clearScreen: false
	}
};



function typeCheck() {
	return {
	  writeBundle() {
		require('child_process').spawn('svelte-check', {
		  stdio: ['ignore', 'inherit', 'inherit'],
		  shell: true
		});
	  }
	}
  }